/*
 * TestSuite.java
 *
 * Created on 25. 12. 2003, 1:16
 */

package saiko.compression;

import java.io.*;
import java.util.*;
import java.security.*;
import java.lang.reflect.*;
/**
 * class for testing the results of compression
 * from the source directory all files in the recursive subdirectory structure
 * are read and coded by given output streams and than back by given input streams, the
 * result is chcecked and report is printed about the file size comparism and ratio.
 * Note: this test suite is certainly not optimized for speed.
 *
 * @author  Dusan Saiko
 */public class TestSuite {
    
    /**
     * class to hold informations about the testing compression alorithms
     */
    class CompressionAlgorithms  {
        Class inputStream;
        Class outputStream;
        String ext;
    
        /**
         * Constructor for the class
         *
         * @param aInputStreamClass (String) - class name to do the decompression
         * @param aOutputStreamCless (String) - class name to do the compression
         * @exception ClassNotFoundException any of the above classes can not be found
         */
        public CompressionAlgorithms(String aInputStreamClass, String aOutputStreamClass, String aExt) 
        throws ClassNotFoundException
        {
            ext = aExt;
            inputStream = Class.forName(aInputStreamClass);
            outputStream = Class.forName(aOutputStreamClass);
        }
    }
    
    private File srcDir;
    private File outputDir;
    private Vector algorithms;
    
    /** 
     * Constructor of new instance of TestSuite
     *
     * @param aSrcDir (String) - directory from which the source files are read
     * @param aOutputDir (String) - directory into which compressed and 
     *        than decompressed files are placed
     * @exception  IOException if an I/O error occurs, 
     *               the aOutputDir does not exists and can not be created
     *               or aSrcDir does not exists
    */    
    public TestSuite(String aSrcDir, String aOutputDir) 
    throws IOException
    {
        srcDir = new File(aSrcDir);
        outputDir=new File(aOutputDir);
        outputDir.mkdirs();
        if(!outputDir.exists()) throw new IOException("Can not write the directory "+aOutputDir);
        if(!srcDir.exists()) throw new IOException("Can not read the directory "+aOutputDir);
        algorithms=new Vector();
    }
    
    /**
     * creates the digests of the content of the file
     *
     * @param aFile (File) - input file for which to get the digest
     * @return String - the hexa MD5 digest of the file
     * @exception IOException if can not read the file
     * @exception NoSuchAlgorithmException if no MD5 is supported in JVM
    */
    private String digest(File aFile) throws IOException, NoSuchAlgorithmException
    {
        MessageDigest md = MessageDigest.getInstance("MD5");
        
        FileInputStream input=new FileInputStream(aFile);
        byte buff[] = new byte[1024];
        int size;
        while((size=input.read(buff))>0) {
            md.update(buff,0,size);
        }
        input.close();
        byte digest[]=md.digest();
        String hash="";
        for(int i=0; i<digest.length; i++) {
            String c=Integer.toHexString(((int)(char)digest[i])&0xFF);
            if(c.length()<2) c="0"+c;
            hash+=c;
        }
        return hash.toUpperCase();
    }

    /**
     * function to add the algorithm to the test set
     *
     * @param aInputStreamClass (String) - class name to do the decompression
     * @param aOutputStreamCless (String) - class name to do the compression
     * @exception ClassNotFoundException any of the above classes can not be found
    */
    public void addAlgorithm(String aInputStreamClass, String aOutputStreamClass, String aExt) 
    throws ClassNotFoundException
    {
       algorithms.add(new CompressionAlgorithms(aInputStreamClass,aOutputStreamClass,aExt));
    }

    /**
     * performs the actual test on the file specified by parameter
     * in the destination directory it creates three files
     * ._SRC - original file to compress
     * .[EXT] - compressed file with exctension stored in aAlg
     * ._DEST - afterward decompressed file
     *
     * @param aAlg (CompressionAlgorithms) - the structure with compression class info
     * @param aOutputDir (File) - output dir to place output files in
     * @param aInput (File) - file or directory from which to read the files to compress
     * @exception IOException if can not read the file
     * @exception NoSuchAlgorithmException if no MD5 is supported in JVM
     * @exception NoSuchMethodException if the aAlg does not contain Input/Output Streams
    */
    private void perform(CompressionAlgorithms aAlg, File aOutputDir, File aInput) 
    throws  IOException, NoSuchAlgorithmException, 
            NoSuchMethodException, InstantiationException, 
            IllegalAccessException, InvocationTargetException
    {
        if(aInput.isDirectory()) {
            File dir = new File(aOutputDir,aInput.getName());
            dir.mkdirs();
            File files[]=aInput.listFiles();
            if(files==null) return;
            for(int f=0; f<files.length; f++) {
                perform(aAlg, dir, files[f]);
            }            
        } else {
            System.out.print("File "+new File(aInput.getParent()).getName()+"/"+aInput.getName()+"  "+aAlg.ext);
            
            long orgFileSize=aInput.length();
            long cmpFileSize;
            long destFileSize;
            
            String orgDigest=digest(aInput);
            String destDigest;
            
            // the test itself
            //copy the original file to the dest directory
            {
                InputStream in=new FileInputStream(aInput);
                File outFile=new File(aOutputDir,aInput.getName()+"._SRC");
                OutputStream out=new FileOutputStream(outFile);
                int i;
                while((i=in.read())!=-1) {
                    out.write(i);
                }
                out.close();
                in.close();
            }
            System.out.print(".");
            // compress the file first
            {
                InputStream in=new FileInputStream(aInput);
                File outFile=new File(aOutputDir,aInput.getName()+"."+aAlg.ext);
                
                Constructor c=aAlg.outputStream.getConstructor(new Class[] { OutputStream.class });
                OutputStream out=(OutputStream) c.newInstance(new Object[] { new FileOutputStream(outFile)});
                int i;
                while((i=in.read())!=-1) {
                    out.write(i);
                }
                out.close();
                in.close();
                cmpFileSize=outFile.length();
            }
            System.out.print(".");
            //recreate the original file
            {
                Constructor c=aAlg.inputStream.getConstructor(new Class[] { InputStream.class });
                InputStream in=(InputStream)c.newInstance(new Object[] {new FileInputStream(new File(aOutputDir,aInput.getName()+"."+aAlg.ext))});
                File outFile=new File(aOutputDir,aInput.getName()+"._DEST");
                OutputStream out=new FileOutputStream(outFile);
                int i;
                while((i=in.read())!=-1) {
                    out.write(i);
                }
                out.close();
                in.close();
                destFileSize=outFile.length();
                destDigest=digest(outFile);
            }
            System.out.print(".");
            
            System.out.print(destDigest.equals(orgDigest) && orgFileSize==destFileSize ? "  OK  " : "  !ERR!  ");
            System.out.print(cmpFileSize+"  "+Math.round((cmpFileSize/(double)orgFileSize)*10000.00)/100.00+"%");
            System.out.println();
        } //is file
    }

    /**
     * function executes the compression tests on the algorithms added by addAlgorithm method
     *
     * @exception  IOException if an I/O error occurs or the aSrcDir does not exists and can not be created
     * @exception NoSuchAlgorithmException if no MD5 is supported in JVM
     * @exception NoSuchMethodException if some of the addAlgorithm parameters were not valid
     */
    public void perform() 
    throws  IOException, NoSuchAlgorithmException, 
            NoSuchMethodException, InstantiationException,
            IllegalAccessException, InvocationTargetException
    {
        perform(0);
    }

    /**
     * function executes the compression tests on the algorithms added by addAlgorithm method
     *
     * @param aCount (int) - number of files/directories to test on (can be used for testing purpose to limit the
     *                      number of files where testing is done), 0 for all files and directories 
     *                      specified in the constructor
     * @exception  IOException if an I/O error occurs or the aSrcDir does not exists and can not be created
     * @exception NoSuchAlgorithmException if no MD5 is supported in JVM
     * @exception NoSuchMethodException if some of the addAlgorithm parameters were not valid
     */
    public void perform(int aCount) 
    throws  IOException, NoSuchAlgorithmException, 
            NoSuchMethodException, InstantiationException,
            IllegalAccessException, InvocationTargetException
    {
        for(int a=0; a<algorithms.size(); a++) {
            CompressionAlgorithms alg = (CompressionAlgorithms) algorithms.get(a);
            
            File dir = new File(outputDir, alg.ext);
            dir.mkdirs();

            File files[]=srcDir.listFiles();
            if(files==null) return;
            int count=files.length;
            if(aCount>0) {
                count=Math.min(count,aCount);
            }
            
            for(int f=0; f<count; f++) {
                perform(alg, dir, files[f]);
            }
        }
    }
    
    
    /** runs the compression test suite */
    public static void main(String aArg[]) throws Exception  {
        TestSuite tests=new TestSuite("d:/bakule/benchmark_test_suite/files","c:/!!");
        
        tests.addAlgorithm("saiko.compression.RLEInputStream", "saiko.compression.RLEOutputStream", "RLE");
        tests.addAlgorithm("saiko.compression.BitMapsInputStream", "saiko.compression.BitMapsOutputStream", "BM");
        tests.addAlgorithm("saiko.compression.ExpandedBitBaseInputStream", "saiko.compression.ExpandedBitBaseOutputStream", "EBB");
        tests.addAlgorithm("saiko.compression.DiatomicInputStream", "saiko.compression.DiatomicOutputStream", "DIA");
        tests.addAlgorithm("saiko.compression.HuffmanInputStream", "saiko.compression.HuffmanOutputStream", "HUF");
        tests.addAlgorithm("com.colloquial.arithcode.ArithCodeInputStream", "com.colloquial.arithcode.ArithCodeOutputStream", "ART");        
        tests.addAlgorithm("java.util.zip.GZIPInputStream", "java.util.zip.GZIPOutputStream", "GZIP");        
        tests.addAlgorithm("org.apache.tools.bzip2.CBZip2InputStream", "org.apache.tools.bzip2.CBZip2OutputStream", "BZIP2");        
        
        tests.perform();
    }
    
}
